package com.soft.nio.connector.core;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.w3c.dom.Element;

import com.mchange.v2.c3p0.ComboPooledDataSource;

/**
 *数据库连接池,需要 mysql-connector.jar和c3p0-0.9.0.2.jar
 */
public class SQLConnect {

	private ComboPooledDataSource cpds = null;
	private static SQLConnect instance;

	public static SQLConnect newInstance() {
		if (instance == null)
			instance = new SQLConnect();
		return instance;
	}

	private SQLConnect() {
		System.out.println("initiate databases config ...");
		try {
			Element element = NetTool.LoadXml("config", "MYSQL");
			String driver = NetTool.getXmlProperty(element, "driver");
			String url = NetTool.getXmlProperty(element, "url");
			String name = NetTool.getXmlProperty(element, "name");
			String pwd = NetTool.getXmlProperty(element, "pwd");
			if (isClosed(driver, url, name, pwd)) {
				System.out.println("mysql connect failed...");
				return;
			}
			cpds = new ComboPooledDataSource();
			// cpds.setDriverClass(driver);
			cpds.setJdbcUrl(url);
			cpds.setUser(name);
			cpds.setPassword(pwd);
			int minConnectionPool = Integer.parseInt(NetTool.getXmlProperty(
					element, "minConnectionPool"));
			System.out.println("minConnectionPool:" + minConnectionPool);
			cpds.setInitialPoolSize(minConnectionPool);
			cpds.setMinPoolSize(minConnectionPool);
			int maxConnectionPool = Integer.parseInt(NetTool.getXmlProperty(
					element, "maxConnectionPool"));
			System.out.println("maxConnectionPool:" + maxConnectionPool);
			cpds.setMaxPoolSize(maxConnectionPool);
			int acquireIncrement = Integer.parseInt(NetTool.getXmlProperty(
					element, "acquireIncrement"));
			cpds.setAcquireIncrement(acquireIncrement);
			int maxIdleTime = Integer.parseInt(NetTool.getXmlProperty(element,
					"maxIdleTime"));
			cpds.setMaxIdleTime(maxIdleTime);
			int maxStatementsAndPerConnection = Integer.parseInt(NetTool
					.getXmlProperty(element, "maxStatementsAndPerConnection"));
			cpds.setMaxStatements(maxStatementsAndPerConnection);
			int helpThread = Integer.parseInt(NetTool.getXmlProperty(element,
					"helpThread"));
			cpds.setNumHelperThreads(helpThread);
			cpds.setAutoCommitOnClose(true);
			cpds.setIdleConnectionTestPeriod(100);
			Connection connection = cpds.getConnection();
			if (connection == null)
				System.out.println("databases is closed...");
			else
				System.out.println("databases is completed ...");
			connection.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void test() {
		try {
			Connection connection = getConnection();
			PreparedStatement ps = connection
					.prepareStatement("select * from user");
			ResultSet rs = ps.executeQuery();
			if (rs.next()) {
				System.out.println(">>>" + rs.getString("name"));
			}
			connection.close();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected boolean isClosed(String driver, String url, String name,
			String pwd) {
		try {
			Class.forName(driver);
			Connection con = DriverManager.getConnection(url, name, pwd);
			if (con.isClosed())
				return true;
			con.close();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return true;
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return true;
		}
		return false;
	}

	public static Connection getConnection() {
		try {
			return newInstance().cpds.getConnection();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			// System.out.println("check mysql is opening ...");
		}
		return null;
	}

}
